using System;
using System.Reflection;

namespace Syntriax.Engine.Core.Factory;

public static class TypeFactory
{
    public static T Get<T>(params object?[]? args) where T : class
        => Get<T>(typeof(T), args);

    public static Type Get(string typeName)
    {
        foreach (var type in Assembly.GetExecutingAssembly().GetTypes())
            if (type.FullName?.Equals(typeName, StringComparison.OrdinalIgnoreCase) ?? false)
                return type;
        throw new Exception();
    }

    public static T Get<T>(Type type, params object?[]? args) where T : class
    {
        T? result;

        if (args is not null && args.Length != 0)
            result = Activator.CreateInstance(type, args) as T;
        else
            result = Activator.CreateInstance(type) as T;

        if (result is null)
            throw new Exception($"{type.Name} could not be created or casted to {typeof(T).Name}.");

        return result;
    }
}
